#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# audioconcat.py - Concatenates multiple audio files to single ogg/vorbis file
# Uses the gnonlin elements (http://gnonlin.sf.net/)
# Copyright (C) 2005 Edward Hervey <edward@fluendo.com>
#               2006 Jason Gerard DeRose <jderose@jasonderose.org>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.
# 

import sys

import gobject
gobject.threads_init()

import pygst
pygst.require('0.10')
import gst
from gst.extend.discoverer import Discoverer



class AudioDec(gst.Bin):
	'''Decodes audio file, outputs at specified caps'''

	def __init__(self, location, caps):
		gst.Bin.__init__(self)

		# Create elements
		src = gst.element_factory_make('filesrc')
		dec = gst.element_factory_make('decodebin')
		conv = gst.element_factory_make('audioconvert')
		rsmpl = gst.element_factory_make('audioresample')
		ident = gst.element_factory_make('identity')

		# Set 'location' property on filesrc
		src.set_property('location', location)

		# Connect handler for 'new-decoded-pad' signal 
		dec.connect('new-decoded-pad', self.__on_new_decoded_pad)

		# Add elements to bin
		self.add(src, dec, conv, rsmpl, ident)

		# Link *some* elements 
		# This is completed in self.__on_new_decoded_pad()
		src.link(dec)
		conv.link(rsmpl)
		rsmpl.link(ident, caps)

		# Reference used in self.__on_new_decoded_pad()
		self.__apad = conv.get_pad('sink')

		# Add ghost pad
		self.add_pad(gst.GhostPad('src', ident.get_pad('src')))


	def __on_new_decoded_pad(self, element, pad, last):
		caps = pad.get_caps()
		name = caps[0].get_name()
		print '\n__on_new_decoded_pad:', name
		if 'audio' in name:
			if not self.__apad.is_linked(): # Only link once
				pad.link(self.__apad)




class AudioConcat:
	'''Concatenates multiple audio files to single ogg/vorbis file'''

	caps = gst.caps_from_string('audio/x-raw-float, rate=44100, channels=2, endianness=1234, width=32')
	
	def __init__(self, infiles, outfile):
		# These are used in iteration through infiles	
		self.infiles = infiles
		self.i = 0
		self.start = 0L

		# The pipeline
		self.pipeline = gst.Pipeline()

		# Create bus and connect 'eos' and 'error' handlers
		self.bus = self.pipeline.get_bus()
		self.bus.add_signal_watch()
		self.bus.connect('message::eos', self.on_eos)
		self.bus.connect('message::error', self.on_error)

		# Create elements
		self.comp = gst.element_factory_make('gnlcomposition')
		self.enc = gst.element_factory_make('vorbisenc')
		self.mux = gst.element_factory_make('oggmux')
		self.sink = gst.element_factory_make('filesink')

		# Connect handler for 'pad-added' signal 
		self.comp.connect('pad-added', self.on_pad_added)	

		# Set 'location' property on filesink
		self.sink.set_property('location', outfile)

		# Add elements to pipeline
		self.pipeline.add(self.comp, self.enc, self.mux, self.sink)

		# Link *some* elements
		# This in completed in self.on_pad_added()
		gst.element_link_many(self.enc, self.mux, self.sink)

		# Reference used in self.on_pad_added()
		self.apad = self.enc.get_pad('sink')

		# The MainLoop
		self.mainloop = gobject.MainLoop()

		# Iterate through infiles
		gobject.idle_add(self.discover)
		self.mainloop.run()


	def discover(self):
		infile = self.infiles[self.i]
		discoverer = Discoverer(infile)
		discoverer.connect('discovered', self.on_discovered, infile)
		discoverer.discover()
		return False # Don't repeat idle call


	def on_discovered(self, discoverer, ismedia, infile):
		print '\non_discovered:', infile
		discoverer.print_info()
		if discoverer.is_audio:
			dec = AudioDec(infile, self.caps)
			src = gst.element_factory_make('gnlsource')
			src.add(dec)
			src.set_property('media-start', 0L)
			src.set_property('media-duration', discoverer.audiolength)
			src.set_property('start', self.start)
			src.set_property('duration', discoverer.audiolength)
			self.comp.add(src)
			self.start += discoverer.audiolength
		self.i += 1
		if self.i < len(self.infiles):
			gobject.idle_add(self.discover)
		else:
			if self.start > 0: # At least 1 infile is_audio and audiolength > 0
				self.pipeline.set_state(gst.STATE_PLAYING)
			else:
				self.mainloop.quit()


	def on_pad_added(self, element, pad):
		caps = pad.get_caps()
		name = caps[0].get_name()
		print '\non_pad_added:', name
		if name == 'audio/x-raw-float':
			if not self.apad.is_linked(): # Only link once
				pad.link(self.apad)


	def on_eos(self, bus, msg):
		print '\non_eos'
		self.mainloop.quit()


	def on_error(self, bus, msg):
		error = msg.parse_error()
		print '\non_error:', error[1]
		self.mainloop.quit()




if __name__ == '__main__':
	if len(sys.argv) >= 3:
		AudioConcat(sys.argv[1:-1], sys.argv[-1])
	else:
		print 'Usage: %s <input_file(s)> <output_file>' % sys.argv[0]
		print 'Example: %s song1.mp3 song2.ogg output.ogg' % sys.argv[0]



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2002 David I. Lehn
# Copyright (C) 2004 Johan Dahlin
# Copyright (C) 2005 Edward Hervey
# Copyright (C) 2005 Thomas Vander Stichele
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

from common import gobject, gst, unittest, TestCase, pygobject_2_13

import sys
import time

# see
# http://www.sicem.biz/personal/lgs/docs/gobject-python/gobject-tutorial.html
class MyBin(gst.Bin):
    _state_changed = False

    def __init__(self, name):
        # we need to call GObject's init to be able to do self.do_*
        gobject.GObject.__init__(self)
        # since we can't chain up to our parent's __init__, we set the
        # name manually 
        self.set_property('name', name)

    def do_change_state(self, state_change):
        if state_change == gst.STATE_CHANGE_PAUSED_TO_PLAYING:
            self._state_changed = True
        # FIXME: it seems a vmethod increases the refcount without unreffing
        # print self.__gstrefcount__
        # print self.__grefcount__

        # chain up to parent
        return gst.Bin.do_change_state(self, state_change)

# we need to register the type for PyGTK < 2.8
gobject.type_register(MyBin)

# FIXME: fix leak in vmethods before removing overriding fixture
class BinSubclassTest(TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testStateChange(self):
        bin = MyBin("mybin")
        self.assertEquals(bin.__gstrefcount__, 1)
        self.assertEquals(sys.getrefcount(bin), pygobject_2_13 and 2 or 3)

        self.assertEquals(bin.get_name(), "mybin")
        self.assertEquals(bin.__gstrefcount__, 1)

        # test get_state with no timeout
        (ret, state, pending) = bin.get_state()
        self.failIfEqual(ret, gst.STATE_CHANGE_FAILURE)
        self.assertEquals(bin.__gstrefcount__, 1)

        # set to playing
        bin.set_state(gst.STATE_PLAYING)
        self.failUnless(bin._state_changed)

        # test get_state with no timeout
        (ret, state, pending) = bin.get_state()
        self.failIfEqual(ret, gst.STATE_CHANGE_FAILURE)

        if ret == gst.STATE_CHANGE_SUCCESS:
            self.assertEquals(state, gst.STATE_PLAYING)
            self.assertEquals(pending, gst.STATE_VOID_PENDING)

        # test get_state with a timeout
        (ret, state, pending) = bin.get_state(1)
        self.failIfEqual(ret, gst.STATE_CHANGE_FAILURE)

        if ret == gst.STATE_CHANGE_SUCCESS:
            self.assertEquals(state, gst.STATE_PLAYING)
            self.assertEquals(pending, gst.STATE_VOID_PENDING)

        (ret, state, pending) = bin.get_state(timeout=gst.SECOND)

        # back to NULL
        bin.set_state(gst.STATE_NULL)

class BinAddRemove(TestCase):
    def setUp(self):
        TestCase.setUp(self)
        self.bin = gst.Bin('bin')

    def tearDown(self):
        del self.bin
        TestCase.tearDown(self)

    def testError(self):
        gst.info("creating fakesrc")
        src = gst.element_factory_make('fakesrc', 'name')
        gst.info("creating fakesink")
        sink = gst.element_factory_make('fakesink', 'name')
        gst.info("adding src:%d to bin" % src.__gstrefcount__)
        self.assertEqual(src.__gstrefcount__, 1)
        self.bin.add(src)
        self.assertEqual(src.__gstrefcount__, 2)
        gst.info("added src:%d" % src.__gstrefcount__)
        self.assertRaises(gst.AddError, self.bin.add, sink)
        self.assertRaises(gst.AddError, self.bin.add, src)
        self.assertRaises(gst.RemoveError, self.bin.remove, sink)
        gst.info("removing src")
        self.bin.remove(src)
        gst.info("removed")
        self.assertRaises(gst.RemoveError, self.bin.remove, src)
        
    def testMany(self):
        src = gst.element_factory_make('fakesrc')
        sink = gst.element_factory_make('fakesink')
        self.bin.add(src, sink)
        self.assertRaises(gst.AddError, self.bin.add, src, sink)
        self.bin.remove(src, sink)
        self.assertRaises(gst.RemoveError, self.bin.remove, src, sink)

class Preroll(TestCase):
    def setUp(self):
        TestCase.setUp(self)
        self.bin = gst.Bin('bin')

    def tearDown(self):
        # FIXME: wait for state change thread to settle down
        while self.bin.__gstrefcount__ > 1:
            time.sleep(0.1)
        self.assertEquals(self.bin.__gstrefcount__, 1)
        del self.bin
        TestCase.tearDown(self)

    def testFake(self):
        src = gst.element_factory_make('fakesrc')
        sink = gst.element_factory_make('fakesink')
        self.bin.add(src)

        # bin will go to paused, src pad task will start and error out
        self.bin.set_state(gst.STATE_PAUSED)
        ret = self.bin.get_state()
        self.assertEquals(ret[0], gst.STATE_CHANGE_SUCCESS)
        self.assertEquals(ret[1], gst.STATE_PAUSED)
        self.assertEquals(ret[2], gst.STATE_VOID_PENDING)

        # adding the sink will cause the bin to go in preroll mode
        gst.debug('adding sink and setting to PAUSED, should cause preroll')
        self.bin.add(sink)
        sink.set_state(gst.STATE_PAUSED)
        ret = self.bin.get_state(timeout=0)
        self.assertEquals(ret[0], gst.STATE_CHANGE_ASYNC)
        self.assertEquals(ret[1], gst.STATE_PAUSED)
        self.assertEquals(ret[2], gst.STATE_PAUSED)

        # to actually complete preroll, we need to link and re-enable fakesrc
        src.set_state(gst.STATE_READY)
        src.link(sink)
        src.set_state(gst.STATE_PAUSED)
        ret = self.bin.get_state()
        self.assertEquals(ret[0], gst.STATE_CHANGE_SUCCESS)
        self.assertEquals(ret[1], gst.STATE_PAUSED)
        self.assertEquals(ret[2], gst.STATE_VOID_PENDING)

        self.bin.set_state(gst.STATE_NULL)
        self.bin.get_state()
 
class ConstructorTest(TestCase):
    def testGood(self):
        bin = gst.Bin()
        bin = gst.Bin(None)
        bin = gst.Bin('')
        bin = gst.Bin('myname')
        
    def testBad(self):
        # these are now valid. pygobject will take care of converting
        # the arguments to a string.
        #self.assertRaises(TypeError, gst.Bin, 0)
        #self.assertRaises(TypeError, gst.Bin, gst.Bin())
        pass
        
if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Segments.py
# Copyright (C) 2006 Artem Popov <artfwo@gmail.com>
#
# This example demonstrates segment seeking
# and seamless looping within playbin.

import pygst
pygst.require ("0.10")
import gst

import pygtk
pygtk.require ("2.0")
import gobject

class Looper (gobject.GObject):
	__gproperties__ = {
		"loop": (gobject.TYPE_BOOLEAN,
			"loop",
			"Whether to loop the segment",
			False,
			gobject.PARAM_READWRITE),
		"start-pos": (gobject.TYPE_UINT64,
			"start position",
			"The segment start marker",
			0,
			0xfffffffffffffff, # max long possible
			0,
			gobject.PARAM_READWRITE),
		"stop-pos": (gobject.TYPE_UINT64,
			"stop position",
			"The segment stop marker",
			0,
			0xfffffffffffffff, # max long possible
			0,
			gobject.PARAM_READWRITE),
	} # __gproperties__
	
	__gsignals__ = {
		"stopped": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
		"position-updated": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_FLOAT,)),
		"error": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,))
	} # __gsignals__
	
	def __init__ (self, location = None):
		gobject.GObject.__init__ (self)
		
		self.__playbin = gst.element_factory_make ("playbin")
		self.__playbin.props.video_sink = gst.element_factory_make ("fakesink")
		
		bus = self.__playbin.get_bus ()
		bus.add_watch (self.__on_bus_message)
		
		self.__loop = False
		self.__start_pos = 0
		self.__stop_pos = 0
		
		self.__timeout_id = 0
		
		if location:
			self.load (location)
	
	def load (self, location):
		self.__playbin.props.uri = location
		self.__start_position = 0
		self.__stop_position = 0
	
	def set_segment (self, start, stop):
		self.props.start_pos = start
		self.props.stop_pos = stop
	
	def play (self):
		if not (self.__start_pos or self.__stop_pos):
			raise RuntimeError, "Cannot start playback, segment was not set!"
		
		self.__playbin.set_state (gst.STATE_PLAYING)
	
	def stop (self, silent = False):
		self.__playbin.set_state (gst.STATE_NULL)
		if not silent:
			self.emit ("stopped")
	
	def do_get_property (self, property):
		if property.name == "loop":
			return self.__loop
		elif property.name == "start-pos":
			return self.__start_pos
		elif property.name == "stop-pos":
			return self.__stop_pos
		else:
			raise AttributeError, "Unknown property %s" % property.name
	
	def do_set_property (self, property, value):
		if property.name == "loop":
			self.__loop = value
		elif property.name == "start-pos":
			self.__start_pos = value
		elif property.name == "stop-pos":
			self.__stop_pos = value
		else:
			raise AttributeError, "Unknown property %s" % property.name
	
	def do_stopped (self):
		if self.__timeout_id:
			gobject.source_remove (self.__timeout_id)
			self.__timeout_id = 0
	
	def __seek (self, start, stop, flush):
		flags = gst.SEEK_FLAG_SEGMENT | gst.SEEK_FLAG_ACCURATE
		if flush:
			flags = flags | gst.SEEK_FLAG_FLUSH
		self.__playbin.seek (1.0, gst.FORMAT_TIME, flags,
			gst.SEEK_TYPE_SET, start,
			gst.SEEK_TYPE_SET, stop)
	
	def __on_timeout (self):
		position = self.__playbin.query_position (gst.FORMAT_TIME) [0]
		self.emit ("position-updated", float (position))
		return True
	
	def __on_bus_message (self, bus, message):
		if message.type == gst.MESSAGE_ERROR:
			error, debug = message.parse_error ()
			self.stop () # this looks neccessary here
			self.emit ("error", (error, debug))
		
		elif message.type == gst.MESSAGE_NEW_CLOCK:
			# we connect the timeout handler here to be sure that further queries succeed
			interval = int ((self.__stop_position - self.__start_position) / (2 * gst.SECOND) + 50)
			self.__timeout_id = gobject.timeout_add (interval, self.__on_timeout)
		
		elif message.type == gst.MESSAGE_STATE_CHANGED:
			old_state, new_state, pending = message.parse_state_changed ()
			if old_state == gst.STATE_READY and new_state == gst.STATE_PAUSED and message.src == self.__playbin:
				self.__seek (self.__start_pos, self.__stop_pos, True)
		
		elif message.type == gst.MESSAGE_SEGMENT_DONE:
			if self.__loop:
				self.__seek (self.__start_pos, self.__stop_pos, False)
			else:
				src = self.__playbin.get_property ("source")
				pad = src.get_pad ('src')
				pad.push_event (gst.event_new_eos ())
				
				# this is the good old way:
				#
				# pads = src.src_pads ()
				# while True:
				# 	try:
				# 		pad = pads.next ()
				# 		pad.push_event (gst.event_new_eos ())
				# 	except:
				# 		break
		
		elif message.type == gst.MESSAGE_EOS:
			self.stop ()
		
		return True

mainloop = gobject.MainLoop ()

def on_looper_stopped (looper):
	mainloop.quit ()

def on_looper_pos_updated (looper, position):
	print round (position / gst.SECOND, 2)

def on_looper_error (looper, error_tuple):
	error, debug = error_tuple
	print "\n\n%s\n\n%s\n\n" % (error, debug)
	mainloop.quit ()

if __name__ == "__main__":
	import sys
	if len (sys.argv) != 5:
		print "Usage: %s <filename|uri> <start_seconds> <stop_seconds> <loop = 0|1>" % sys.argv [0]
		sys.exit (1)
	
	if "://" in sys.argv [1]:
		uri = sys.argv [1]
	else:
		import os.path
		uri = "file://" + os.path.abspath (sys.argv [1])
	
	looper = Looper (uri)
	
	looper.props.start_pos = long (sys.argv [2]) * gst.SECOND
	looper.props.stop_pos = long (sys.argv [3]) * gst.SECOND
	looper.props.loop = int (sys.argv [4])
	
	looper.connect ("stopped", on_looper_stopped)
	looper.connect ("position-updated", on_looper_pos_updated)
	looper.connect ("error", on_looper_error)
	
	looper.play ()
	mainloop.run ()



#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# GStreamer python bindings
# Copyright (C) 2002 David I. Lehn <dlehn@users.sourceforge.net>
#               2004 Johan Dahlin  <johan@gnome.org>
#
# filesrc.py: implements a file source element completely in python
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.

import sys
import gobject; gobject.threads_init()
import pygst
pygst.require('0.10')
import gst

class FileSource(gst.BaseSrc):
    __gsttemplates__ = (
        gst.PadTemplate("src",
                        gst.PAD_SRC,
                        gst.PAD_ALWAYS,
                        gst.caps_new_any()),
        )

    blocksize = 4096
    fd = None
    
    def __init__(self, name):
        self.__gobject_init__()
        self.curoffset = 0
        self.set_name(name)
            
    def set_property(self, name, value):
        if name == 'location':
            self.fd = open(value, 'r')

    def do_create(self, offset, size):
        if offset != self.curoffset:
            self.fd.seek(offset, 0)
        data = self.fd.read(self.blocksize)
        if data:
            self.curoffset += len(data)
            return gst.FLOW_OK, gst.Buffer(data)
        else:
            return gst.FLOW_UNEXPECTED, None
gobject.type_register(FileSource)

def main(args):
    if len(args) != 3:
        print 'Usage: %s input output' % (args[0])
        return -1
    
    bin = gst.Pipeline('pipeline')

    filesrc = FileSource('filesource')
    assert filesrc
    filesrc.set_property('location', args[1])
   
    filesink = gst.element_factory_make('filesink', 'sink')
    filesink.set_property('location', args[2])

    bin.add(filesrc, filesink)
    gst.element_link_many(filesrc, filesink)
    
    bin.set_state(gst.STATE_PLAYING);
    mainloop = gobject.MainLoop()

    def bus_event(bus, message):
        t = message.type
        if t == gst.MESSAGE_EOS:
            mainloop.quit()
        elif t == gst.MESSAGE_ERROR:
            err, debug = message.parse_error()
            print "Error: %s" % err, debug
            mainloop.quit()           
        return True
    bin.get_bus().add_watch(bus_event)

    mainloop.run()
    bin.set_state(gst.STATE_NULL)

if __name__ == '__main__':
   sys.exit(main(sys.argv))




#!/usr/bin/env python3
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2002 David I. Lehn
# Copyright (C) 2004 Johan Dahlin
# Copyright (C) 2005 Edward Hervey
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

import os
import sys
import unittest


def _testcases(filenames):
    """Yield testcases out of filenames."""
    for filename in filenames:
        if filename.endswith(".py"):
            yield filename[:-3]


def _tests_suite():
    """Pick which tests to run."""
    testcase = os.getenv("TESTCASE")
    if testcase:
        testcases = [testcase]
    else:
        testcases = _testcases(sys.argv[1:])
    loader = unittest.TestLoader()
    return loader.loadTestsFromNames(testcases)


def setup():
    return


if __name__ == "__main__":
    setup()

    # Set verbosity.
    descriptions = 1
    verbosity = 1
    if 'VERBOSE' in os.environ:
        descriptions = 2
        verbosity = 2

    suite = _tests_suite()
    if not list(suite):
        raise Exception("No tests found")

    # Run the tests.
    testRunner = unittest.TextTestRunner(descriptions=descriptions,
                                         verbosity=verbosity)
    result = testRunner.run(suite)
    if result.failures or result.errors:
        sys.exit(1)



#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# gst-python
# Copyright (C) 2005 Fluendo S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.
#
# Author: Andy Wingo <wingo@pobox.com>

import gtk
from gtk import gdk
import gobject

import pygst
pygst.require('0.10')
import gst

class DebugSlider(gtk.HScale):
    def __init__(self):
        adj = gtk.Adjustment(int(gst.debug_get_default_threshold()),
                             0, 5, 1, 0, 0)
        gtk.HScale.__init__(self, adj)
        self.set_digits(0)
        self.set_draw_value(True)
        self.set_value_pos(gtk.POS_TOP)

        def value_changed(self):
            newlevel = int(self.get_adjustment().get_value())
            gst.debug_set_default_threshold(newlevel)

        self.connect('value-changed', value_changed)

if __name__ == '__main__':
    p = gst.parse_launch('fakesrc ! fakesink')
    p.set_state(gst.STATE_PLAYING)

    w = gtk.Window()
    s = DebugSlider()
    w.add(s)
    s.show()
    w.set_default_size(200, 40)
    w.show()
    w.connect('delete-event', lambda *args: gtk.main_quit())
    gtk.main()



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Copyright (C) 2009 Thomas Vander Stichele
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

import sys
import overrides_hack
overrides_hack
from common import TestCase, unittest

from gi.repository import Gst

class TimeArgsTest(TestCase):
    def testNoneTime(self):
        self.assertRaises(TypeError, Gst.TIME_ARGS, None)

    def testStringTime(self):
        self.assertRaises(TypeError, Gst.TIME_ARGS, "String")

    def testClockTimeNone(self):
        self.assertEquals(Gst.TIME_ARGS(Gst.CLOCK_TIME_NONE), 'CLOCK_TIME_NONE')

    def testOneSecond(self):
        self.assertEquals(Gst.TIME_ARGS(Gst.SECOND), '0:00:01.000000000')

class TestNotInitialized(TestCase):
    def testNotInitialized(self):
        if sys.version_info >= (3, 0):
            assert_type = Gst.NotInitialized
        else:
            assert_type = TypeError

        with self.assertRaises(assert_type):
            Gst.Caps.from_string("audio/x-raw")

        with self.assertRaises(assert_type):
            Gst.Structure.from_string("audio/x-raw")

        with self.assertRaises(assert_type):
            Gst.ElementFactory.make("identity", None)

    def testNotDeinitialized(self):
        Gst.init(None)

        assert(Gst.Caps.from_string("audio/x-raw"))
        assert(Gst.Structure.from_string("audio/x-raw"))
        assert(Gst.ElementFactory.make("identity", None))

        Gst.deinit()
        if sys.version_info >= (3, 0):
            assert_type = Gst.NotInitialized
        else:
            assert_type = TypeError

        with self.assertRaises(assert_type):
            Gst.Caps.from_string("audio/x-raw")

        with self.assertRaises(assert_type):
            Gst.Structure.from_string("audio/x-raw")

        with self.assertRaises(assert_type):
            Gst.ElementFactory.make("identity", None)


class TestStructure(TestCase):

    def test_new(self):
        Gst.init(None)
        test = Gst.Structure('test', test=1)
        self.assertEqual(test['test'], 1)

        test = Gst.Structure('test,test=1')
        self.assertEqual(test['test'], 1)


class TestBin(TestCase):

    def test_add_pad(self):
        Gst.init(None)
        self.assertEqual(Gst.ElementFactory.make("bin", None).sinkpads, [])

class TestBufferMap(TestCase):

    def test_map_unmap_manual(self):
        Gst.init(None)
        buf = Gst.Buffer.new_wrapped([42])
        info = buf.map(Gst.MapFlags.READ | Gst.MapFlags.WRITE)
        self.assertEqual(info.data[0], 42)
        buf.unmap(info)
        with self.assertRaises(ValueError):
            info.data[0]

    def test_map_unmap_context(self):
        Gst.init(None)
        buf = Gst.Buffer.new_wrapped([42])
        with buf.map(Gst.MapFlags.READ | Gst.MapFlags.WRITE) as info:
            self.assertEqual(info.data[0], 42)
        with self.assertRaises(ValueError):
            info.data[0]

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

'''
Simple example to demonstrate dynamically adding and removing source elements
to a playing pipeline.
'''

import sys
import random

import gi
gi.require_version('Gst', '1.0')
gi.require_version('GLib', '2.0')
gi.require_version('GObject', '2.0')
from gi.repository import GLib, GObject, Gst

class ProbeData:
    def __init__(self, pipe, src):
        self.pipe = pipe
        self.src = src

def bus_call(bus, message, loop):
    t = message.type
    if t == Gst.MessageType.EOS:
        sys.stdout.write("End-of-stream\n")
        loop.quit()
    elif t == Gst.MessageType.ERROR:
        err, debug = message.parse_error()
        sys.stderr.write("Error: %s: %s\n" % (err, debug))
        loop.quit()
    return True

def dispose_src_cb(src):
    src.set_state(Gst.State.NULL)

def probe_cb(pad, info, pdata):
    peer = pad.get_peer()
    pad.unlink(peer)
    pdata.pipe.remove(pdata.src)
    # Can't set the state of the src to NULL from its streaming thread
    GLib.idle_add(dispose_src_cb, pdata.src)

    pdata.src = Gst.ElementFactory.make('videotestsrc')
    pdata.src.props.pattern = random.randint(0, 24)
    pdata.pipe.add(pdata.src)
    srcpad = pdata.src.get_static_pad ("src")
    srcpad.link(peer)
    pdata.src.sync_state_with_parent()

    GLib.timeout_add_seconds(1, timeout_cb, pdata)

    return Gst.PadProbeReturn.REMOVE

def timeout_cb(pdata):
    srcpad = pdata.src.get_static_pad('src')
    srcpad.add_probe(Gst.PadProbeType.IDLE, probe_cb, pdata)
    return GLib.SOURCE_REMOVE

def main(args):
    GObject.threads_init()
    Gst.init(None)

    pipe = Gst.Pipeline.new('dynamic')
    src = Gst.ElementFactory.make('videotestsrc')
    sink = Gst.ElementFactory.make('autovideosink')
    pipe.add(src, sink)
    src.link(sink)

    pdata = ProbeData(pipe, src)

    loop = GObject.MainLoop()

    GLib.timeout_add_seconds(1, timeout_cb, pdata)

    bus = pipe.get_bus()
    bus.add_signal_watch()
    bus.connect ("message", bus_call, loop)
    
    # start play back and listen to events
    pipe.set_state(Gst.State.PLAYING)
    try:
      loop.run()
    except:
      pass
    
    # cleanup
    pipe.set_state(Gst.State.NULL)

if __name__ == '__main__':
    sys.exit(main(sys.argv))



#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# sinkelement.py
# (c) 2005 Edward Hervey <edward@fluendo.com>
# (c) 2007 Jan Schmidt <jan@fluendo.com>
# Licensed under LGPL
#
# Small test application to show how to write a sink element
# in 20 lines in python and place into the gstreamer registry
# so it can be autoplugged or used from parse_launch.
#
# Run this script with GST_DEBUG=python:5 to see the debug
# messages

import pygst
pygst.require('0.10')
import gst
import gobject
gobject.threads_init ()

#
# Simple Sink element created entirely in python
#

class MySink(gst.Element):
    __gstdetails__ = ('CustomSink','Sink', \
                      'Custom test sink element', 'Edward Hervey')

    _sinkpadtemplate = gst.PadTemplate ("sinkpadtemplate",
                                        gst.PAD_SINK,
                                        gst.PAD_ALWAYS,
                                        gst.caps_new_any())

    def __init__(self):
        gst.Element.__init__(self)
        gst.info('creating sinkpad')
        self.sinkpad = gst.Pad(self._sinkpadtemplate, "sink")
        gst.info('adding sinkpad to self')
        self.add_pad(self.sinkpad)

        gst.info('setting chain/event functions')
        self.sinkpad.set_chain_function(self.chainfunc)
        self.sinkpad.set_event_function(self.eventfunc)
        
    def chainfunc(self, pad, buffer):
        self.info("%s timestamp(buffer):%d" % (pad, buffer.timestamp))
        return gst.FLOW_OK

    def eventfunc(self, pad, event):
        self.info("%s event:%r" % (pad, event.type))
        return True

gobject.type_register(MySink)

# Register the element into this process' registry.
gst.element_register (MySink, 'mysink', gst.RANK_MARGINAL)

print "Use --gst-debug=python:3 to see output from this example"

#
# Code to test the MySink class
#
gst.info('About to create MySink')
pipeline = gst.parse_launch ("fakesrc ! mysink")
pipeline.set_state(gst.STATE_PLAYING)

gobject.MainLoop().run()



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2002 David I. Lehn
# Copyright (C) 2004 Johan Dahlin
# Copyright (C) 2005 Edward Hervey
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

import sys
from common import gst, unittest, TestCase

class StructureTest(TestCase):
    def setUp(self):
        TestCase.setUp(self)
        self.struct = gst.structure_from_string('video/x-raw-yuv,width=10,foo="bar",pixel-aspect-ratio=1/2,framerate=5/1,boolean=(boolean)true')

    def testName(self):
        assert self.struct.get_name() == 'video/x-raw-yuv'
        self.struct.set_name('foobar')
        assert self.struct.get_name() == 'foobar'
        
    def testInt(self):
        assert self.struct.has_key('width')
        assert isinstance(self.struct['width'], int)
        assert self.struct['width'] == 10, self.struct['width']
        self.struct['width'] = 5
        assert self.struct.has_key('width')
        assert isinstance(self.struct['width'], int)
        assert self.struct['width'] == 5, self.struct['width']

    def testString(self):
        assert self.struct.has_key('foo')
        assert isinstance(self.struct['foo'], unicode)
        assert self.struct['foo'] == 'bar', self.struct['foo']
        self.struct['foo'] = 'baz'
        assert self.struct.has_key('foo')
        assert isinstance(self.struct['foo'], unicode)
        assert self.struct['foo'] == 'baz', self.struct['foo']

    def testBoolean(self):
        assert self.struct.has_key('boolean')
        assert isinstance(self.struct['boolean'], bool)
        assert self.struct['boolean'] == True, self.struct['boolean']
        self.struct['boolean'] = False
        assert self.struct.has_key('boolean')
        assert isinstance(self.struct['boolean'], bool)
        assert self.struct['boolean'] == False, self.struct['boolean']

    def testCreateInt(self):
        self.struct['integer'] = 5
        assert self.struct.has_key('integer')
        assert isinstance(self.struct['integer'], int)
        assert self.struct['integer'] == 5, self.struct['integer']
        
    def testGstValue(self):
        s = self.struct
        s['fourcc'] = gst.Fourcc('XVID')
        assert s['fourcc'].fourcc == 'XVID'
        s['frac'] = gst.Fraction(3,4)
        assert s['frac'].num == 3
        assert s['frac'].denom == 4
        s['fracrange'] = gst.FractionRange(gst.Fraction(0,1),
                                           gst.Fraction(25,3))
        assert s['fracrange'].low.num == 0
        assert s['fracrange'].low.denom == 1
        assert s['fracrange'].high.num == 25
        assert s['fracrange'].high.denom == 3
        s['intrange'] = gst.IntRange(5,21)
        assert s['intrange'].low == 5
        assert s['intrange'].high == 21
        s['doublerange'] = gst.DoubleRange(6.,21.)
        assert s['doublerange'].low == 6.
        assert s['doublerange'].high == 21.
        s['fixedlist'] = (4, 5, 6)
        assert isinstance(s['fixedlist'], tuple)
        assert s['fixedlist'] == (4, 5, 6)
        s['list'] = [4, 5, 6]
        assert isinstance(s['list'], list)
        assert s['list'] == [4, 5, 6]
        s['boolean'] = True
        assert isinstance(s['boolean'], bool)
        assert s['boolean'] == True
        
        # finally, some recursive tests
        s['rflist'] = ([(['a', 'b'], ['c', 'd']),'e'], ['f', 'g'])
        assert s['rflist'] == ([(['a', 'b'], ['c', 'd']),'e'], ['f', 'g'])
        s['rlist'] = [([(['a', 'b'], ['c', 'd']),'e'], ['f', 'g']), 'h']
        assert s['rlist'] == [([(['a', 'b'], ['c', 'd']),'e'], ['f', 'g']), 'h']

    def testStructureChange(self):
        assert self.struct['framerate'] == gst.Fraction(5, 1)
        self.struct['framerate'] = gst.Fraction(10, 1)
        assert self.struct['framerate'] == gst.Fraction(10, 1)
        self.struct['pixel-aspect-ratio'] = gst.Fraction(4, 2)
        assert self.struct['pixel-aspect-ratio'].num == 2
        assert self.struct['pixel-aspect-ratio'].denom == 1

    def testKeys(self):
        k = self.struct.keys()
        self.failUnless(k)
        self.assertEquals(len(k), 5)
        self.failUnless("width" in k)
        self.failUnless("foo" in k)
        self.failUnless("framerate" in k)
        self.failUnless("pixel-aspect-ratio" in k)
        self.failUnless("boolean" in k)
 
if __name__ == "__main__":
    unittest.main()



'''
Element that generates a sine audio wave with the specified frequency

Requires numpy

Example pipeline:

gst-launch-1.0 py_audiotestsrc ! autoaudiosink
'''

import gi

gi.require_version('Gst', '1.0')
gi.require_version('GstBase', '1.0')
gi.require_version('GstAudio', '1.0')

from gi.repository import Gst, GLib, GObject, GstBase, GstAudio

try:
    import numpy as np
except ImportError:
    Gst.error('py_audiotestsrc requires numpy')
    raise

OCAPS = Gst.Caps.from_string (
        'audio/x-raw, format=F32LE, layout=interleaved, rate=44100, channels=2')

SAMPLESPERBUFFER = 1024

DEFAULT_FREQ = 440
DEFAULT_VOLUME = 0.8
DEFAULT_MUTE = False
DEFAULT_IS_LIVE = False

class AudioTestSrc(GstBase.BaseSrc):
    __gstmetadata__ = ('CustomSrc','Src', \
                      'Custom test src element', 'Mathieu Duponchelle')

    __gproperties__ = {
        "freq": (int,
                 "Frequency",
                 "Frequency of test signal",
                 1,
                 GLib.MAXINT,
                 DEFAULT_FREQ,
                 GObject.ParamFlags.READWRITE
                ),
        "volume": (float,
                   "Volume",
                   "Volume of test signal",
                   0.0,
                   1.0,
                   DEFAULT_VOLUME,
                   GObject.ParamFlags.READWRITE
                  ),
        "mute": (bool,
                 "Mute",
                 "Mute the test signal",
                 DEFAULT_MUTE,
                 GObject.ParamFlags.READWRITE
                ),
        "is-live": (bool,
                 "Is live",
                 "Whether to act as a live source",
                 DEFAULT_IS_LIVE,
                 GObject.ParamFlags.READWRITE
                ),
    }

    __gsttemplates__ = Gst.PadTemplate.new("src",
                                           Gst.PadDirection.SRC,
                                           Gst.PadPresence.ALWAYS,
                                           OCAPS)

    def __init__(self):
        GstBase.BaseSrc.__init__(self)
        self.info = GstAudio.AudioInfo()

        self.freq = DEFAULT_FREQ
        self.volume = DEFAULT_VOLUME
        self.mute = DEFAULT_MUTE

        self.set_live(DEFAULT_IS_LIVE)
        self.set_format(Gst.Format.TIME)

    def do_set_caps(self, caps):
        self.info.from_caps(caps)
        self.set_blocksize(self.info.bpf * SAMPLESPERBUFFER)
        return True

    def do_get_property(self, prop):
        if prop.name == 'freq':
            return self.freq
        elif prop.name == 'volume':
            return self.volume
        elif prop.name == 'mute':
            return self.mute
        elif prop.name == 'is-live':
            return self.is_live
        else:
            raise AttributeError('unknown property %s' % prop.name)

    def do_set_property(self, prop, value):
        if prop.name == 'freq':
            self.freq = value
        elif prop.name == 'volume':
            self.volume = value
        elif prop.name == 'mute':
            self.mute = value
        elif prop.name == 'is-live':
            self.set_live(value)
        else:
            raise AttributeError('unknown property %s' % prop.name)

    def do_start (self):
        self.next_sample = 0
        self.next_byte = 0
        self.next_time = 0
        self.accumulator = 0
        self.generate_samples_per_buffer = SAMPLESPERBUFFER

        return True

    def do_gst_base_src_query(self, query):
        if query.type == Gst.QueryType.LATENCY:
            latency = Gst.util_uint64_scale_int(self.generate_samples_per_buffer,
                    Gst.SECOND, self.info.rate)
            is_live = self.is_live
            query.set_latency(is_live, latency, Gst.CLOCK_TIME_NONE)
            res = True
        else:
            res = GstBase.BaseSrc.do_query(self, query)
        return res

    def do_get_times(self, buf):
        end = 0
        start = 0
        if self.is_live:
            ts = buf.pts
            if ts != Gst.CLOCK_TIME_NONE:
                duration = buf.duration
                if duration != Gst.CLOCK_TIME_NONE:
                    end = ts + duration
                start = ts
        else:
            start = Gst.CLOCK_TIME_NONE
            end = Gst.CLOCK_TIME_NONE

        return start, end

    def do_fill(self, offset, length, buf):
        if length == -1:
            samples = SAMPLESPERBUFFER
        else:
            samples = int(length / self.info.bpf)

        self.generate_samples_per_buffer = samples

        bytes_ = samples * self.info.bpf

        next_sample = self.next_sample + samples
        next_byte = self.next_byte + bytes_
        next_time = Gst.util_uint64_scale_int(next_sample, Gst.SECOND, self.info.rate)

        try:
            with buf.map(Gst.MapFlags.WRITE) as info:
                array = np.ndarray(shape = self.info.channels * samples, dtype = np.float32, buffer = info.data)
                if not self.mute:
                    r = np.repeat(np.arange(self.accumulator, self.accumulator + samples),
                            self.info.channels)
                    np.sin(2 * np.pi * r * self.freq / self.info.rate, out=array)
                    array *= self.volume
                else:
                    array[:] = 0
        except Exception as e:
            Gst.error("Mapping error: %s" % e)
            return Gst.FlowReturn.ERROR

        buf.offset = self.next_sample
        buf.offset_end = next_sample
        buf.pts = self.next_time
        buf.duration = next_time - self.next_time

        self.next_time = next_time
        self.next_sample = next_sample
        self.next_byte = next_byte
        self.accumulator += samples
        self.accumulator %= self.info.rate / self.freq

        return (Gst.FlowReturn.OK, buf)


__gstelementfactory__ = ("py_audiotestsrc", Gst.Rank.NONE, AudioTestSrc)



#!/usr/bin/env python

import sys

import gobject
gobject.threads_init()

import pygst
pygst.require('0.10')
import gst

def bus_call(bus, message, loop):
    t = message.type
    if t == gst.MESSAGE_EOS:
        sys.stout.write("End-of-stream\n")
        loop.quit()
    elif t == gst.MESSAGE_ERROR:
        err, debug = message.parse_error()
        sys.stderr.write("Error: %s: %s\n" % err, debug)
        loop.quit()
    return True

def main(args):
    if len(args) != 2:
        sys.stderr.write("usage: %s <media file or uri>\n" % args[0])
        sys.exit(1)
        
    playbin = gst.element_factory_make("playbin2", None)
    if not playbin:
        sys.stderr.write("'playbin2' gstreamer plugin missing\n")
        sys.exit(1)

    # take the commandline argument and ensure that it is a uri
    if gst.uri_is_valid(args[1]):
      uri = args[1]
    else:
      uri = gst.filename_to_uri(args[1])
    playbin.set_property('uri', uri)

    # create and event loop and feed gstreamer bus mesages to it
    loop = gobject.MainLoop()

    bus = playbin.get_bus()
    bus.add_watch(bus_call, loop)
    
    # start play back and listed to events
    playbin.set_state(gst.STATE_PLAYING)
    loop.run()
    
    # cleanup
    playbin.set_state(gst.STATE_NULL)

if __name__ == '__main__':
    sys.exit(main(sys.argv))



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2002 David I. Lehn
# Copyright (C) 2004 Johan Dahlin
# Copyright (C) 2005 Edward Hervey
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

from common import gst, unittest, TestCase, pygobject_2_13

import sys
import gc
import gobject

class SrcBin(gst.Bin):
    def prepare(self):
        src = gst.element_factory_make('fakesrc')
        self.add(src)
        pad = src.get_pad("src")
        ghostpad = gst.GhostPad("src", pad)
        self.add_pad(ghostpad)
gobject.type_register(SrcBin)

class SinkBin(gst.Bin):
    def prepare(self):
        sink = gst.element_factory_make('fakesink')
        self.add(sink)
        pad = sink.get_pad("sink")
        ghostpad = gst.GhostPad("sink", pad)
        self.add_pad(ghostpad)
        self.sink = sink

    def connect_handoff(self, cb, *args, **kwargs):
        self.sink.set_property('signal-handoffs', True)
        self.sink.connect('handoff', cb, *args, **kwargs)
        
gobject.type_register(SinkBin)

        
class PipeTest(TestCase):
    def setUp(self):
        gst.info("setUp")
        TestCase.setUp(self)
        self.pipeline = gst.Pipeline()
        self.assertEquals(self.pipeline.__gstrefcount__, 1)
        self.assertEquals(sys.getrefcount(self.pipeline), pygobject_2_13 and 2 or 3)

        self.src = SrcBin()
        self.src.prepare()
        self.sink = SinkBin()
        self.sink.prepare()
        self.assertEquals(self.src.__gstrefcount__, 1)
        self.assertEquals(sys.getrefcount(self.src), pygobject_2_13 and 2 or 3)
        self.assertEquals(self.sink.__gstrefcount__, 1)
        self.assertEquals(sys.getrefcount(self.sink), pygobject_2_13 and 2 or 3)
        gst.info("end of SetUp")

    def tearDown(self):
        gst.info("tearDown")
        self.assertTrue (self.pipeline.__gstrefcount__ >= 1 and self.pipeline.__gstrefcount__ <= 2)
        self.assertEquals(sys.getrefcount(self.pipeline), pygobject_2_13 and 2 or 3)
        self.assertEquals(self.src.__gstrefcount__, 2)
        self.assertEquals(sys.getrefcount(self.src), pygobject_2_13 and 2 or 3)
        self.assertEquals(self.sink.__gstrefcount__, 2)
        self.assertEquals(sys.getrefcount(self.sink), 3)
        gst.debug('deleting pipeline')
        del self.pipeline
        self.gccollect()

        self.assertEquals(self.src.__gstrefcount__, 1) # parent gone
        self.assertEquals(self.sink.__gstrefcount__, 1) # parent gone
        self.assertEquals(sys.getrefcount(self.src), pygobject_2_13 and 2 or 3)
        self.assertEquals(sys.getrefcount(self.sink), pygobject_2_13 and 2 or 3)
        gst.debug('deleting src')
        del self.src
        self.gccollect()
        gst.debug('deleting sink')
        del self.sink
        self.gccollect()

        TestCase.tearDown(self)
        
    def testBinState(self):
        self.pipeline.add(self.src, self.sink)
        self.src.link(self.sink)
        self.sink.connect_handoff(self._sink_handoff_cb)
        self._handoffs = 0

        self.assertTrue(self.pipeline.set_state(gst.STATE_PLAYING) != gst.STATE_CHANGE_FAILURE)
        while True:
            (ret, cur, pen) = self.pipeline.get_state()
            if ret == gst.STATE_CHANGE_SUCCESS and cur == gst.STATE_PLAYING:
                break

        while self._handoffs < 10:
                pass

        self.assertEquals(self.pipeline.set_state(gst.STATE_NULL), gst.STATE_CHANGE_SUCCESS)
        while True:
            (ret, cur, pen) = self.pipeline.get_state()
            if ret == gst.STATE_CHANGE_SUCCESS and cur == gst.STATE_NULL:
                break

##     def testProbedLink(self):
##         self.pipeline.add(self.src)
##         pad = self.src.get_pad("src")
        
##         self.sink.connect_handoff(self._sink_handoff_cb)
##         self._handoffs = 0

##         # FIXME: adding a probe to the ghost pad does not work atm
##         # id = pad.add_buffer_probe(self._src_buffer_probe_cb)
##         realpad = pad.get_target()
##         self._probe_id = realpad.add_buffer_probe(self._src_buffer_probe_cb)

##         self._probed = False
        
##         while True:
##             (ret, cur, pen) = self.pipeline.get_state()
##             if ret == gst.STATE_CHANGE_SUCCESS and cur == gst.STATE_PLAYING:
##                 break

##         while not self._probed:
##             pass

##         while self._handoffs < 10:
##             pass

##         self.pipeline.set_state(gst.STATE_NULL)
##         while True:
##             (ret, cur, pen) = self.pipeline.get_state()
##             if ret == gst.STATE_CHANGE_SUCCESS and cur == gst.STATE_NULL:
##                 break

    def _src_buffer_probe_cb(self, pad, buffer):
        gst.debug("received probe on pad %r" % pad)
        self._probed = True
        gst.debug('adding sink bin')
        self.pipeline.add(self.sink)
        # this seems to get rid of the warnings about pushing on an unactivated
        # pad
        gst.debug('setting sink state')
        
        # FIXME: attempt one: sync to current pending state of bin
        (res, cur, pen) = self.pipeline.get_state(timeout=0)
        target = pen
        if target == gst.STATE_VOID_PENDING:
            target = cur
        gst.debug("setting sink state to %r" % target)
        # FIXME: the following print can cause a lock-up; why ?
        # print target
        # if we don't set async, it will possibly end up in PAUSED
        self.sink.set_state(target)
        
        gst.debug('linking')
        self.src.link(self.sink)
        gst.debug('removing buffer probe id %r' % self._probe_id)
        pad.remove_buffer_probe(self._probe_id)
        self._probe_id = None
        gst.debug('done')

    def _sink_handoff_cb(self, sink, buffer, pad):
        gst.debug('received handoff on pad %r' % pad)
        self._handoffs += 1

class TargetTest(TestCase):
    def test_target(self):
        src = gst.Pad("src", gst.PAD_SRC)

        ghost = gst.GhostPad("ghost_src", src)
        self.failUnless(ghost.get_target() is src)

        ghost.set_target(None)
        self.failUnless(ghost.get_target() is None)

        ghost.set_target(src)
        self.failUnless(ghost.get_target() is src)

if __name__ == "__main__":
    unittest.main()



import os
import sys
import imp

class GstOverrideImport:
    def find_module(self, fullname, path=None):
        if fullname in ('gi.overrides.Gst', 'gi.overrides._gi_gst'):
            return self
        return None

    def load_module(self, name):
        if name in sys.modules:
            return sys.modules[name]

        fp, pathname, description = imp.find_module(name.split('.')[-1], [
            os.environ.get('GST_OVERRIDE_SRC_PATH'),
            os.environ.get('GST_OVERRIDE_BUILD_PATH'),
        ])

        try:
            module = imp.load_module(name, fp, pathname, description)
        finally:
            if fp:
                fp.close()
        sys.modules[name] = module
        return module

if sys.version_info.major >= 3:
    sys.meta_path.insert(0, GstOverrideImport())
else:
    import gi.overrides

    gi.overrides.__path__.append(os.environ.get('GST_OVERRIDE_SRC_PATH'))
    gi.overrides.__path__.append(os.environ.get('GST_OVERRIDE_BUILD_PATH'))



#!/usr/bin/env python3

'''
Simple example to demonstrate using Gst.DeviceMonitor and `transcodebin` to
record audio from a microphone into an .oga file
'''
import gi
import sys
gi.require_version('Gst', '1.0')
gi.require_version('GstPbutils', '1.0')
gi.require_version('GLib', '2.0')
gi.require_version('GObject', '2.0')
from gi.repository import GLib, GObject, Gst, GstPbutils

def bus_call(bus, message, loop):
    t = message.type
    Gst.debug_bin_to_dot_file_with_ts(pipeline, Gst.DebugGraphDetails.ALL, "test")
    if t == Gst.MessageType.EOS:
        sys.stdout.write("End-of-stream\n")
        loop.quit()
    elif t == Gst.MessageType.ERROR:
        err, debug = message.parse_error()
        sys.stderr.write("Error: %s: %s\n" % (err, debug))
        loop.quit()
    return True

def stop(loop, pipeline):
    _, position = pipeline.query_position(Gst.Format.TIME)
    print("Position: %s\r" % Gst.TIME_ARGS(position))

    if position > 10 * Gst.SECOND:
        loop.quit()
        print("Stopping after 10 seconds")
        return False

    return True

if __name__ == "__main__":
    Gst.init(sys.argv)

    if len(sys.argv) != 2:
        print("Missing <output-location> parametter")
        sys.exit(1)

    monitor = Gst.DeviceMonitor.new()
    monitor.add_filter("Audio/Source", None)
    monitor.start()

    # This is happening synchonously, use the GstBus based API and
    # monitor.start() to avoid blocking the main thread.
    devices = monitor.get_devices()

    if not devices:
        print("No microphone found...")
        sys.exit(1)

    default = [d for d in devices if d.get_properties().get_value("is-default") is True]
    if len(default) == 1:
        device = default[0]
    else:
        print("Avalaible microphones:")
        for i, d in enumerate(devices):
            print("%d - %s" % (i, d.get_display_name()))
        res = int(input("Select device: "))
        device = devices[res]

    pipeline = Gst.ElementFactory.make("pipeline", None)
    source = device.create_element()
    transcodebin = Gst.ElementFactory.make("transcodebin", None)
    Gst.util_set_object_arg(transcodebin, "profile", "video/ogg:audio/x-opus")
    filesink = Gst.ElementFactory.make("filesink", None)
    filesink.props.location = sys.argv[1]

    pipeline.add(source, transcodebin, filesink)
    source.link(transcodebin)
    transcodebin.link(filesink)

    pipeline.set_state(Gst.State.PLAYING)

    bus = pipeline.get_bus()
    bus.add_signal_watch()

    loop = GLib.MainLoop()
    GLib.timeout_add_seconds(1, stop, loop, pipeline)
    bus.connect ("message", bus_call, loop)
    loop.run()

    pipeline.set_state(Gst.State.NULL)
    pipeline.get_state(Gst.CLOCK_TIME_NONE)


#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# gst-python
# Copyright (C) 2003 David I. Lehn
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.
# 
# Author: David I. Lehn <dlehn@users.sourceforge.net>
#

import pygtk
pygtk.require('2.0')

import sys
import time
import gobject
import gtk

import pygst
pygst.require('0.10')

import gst

class BPS(object):
    def __init__(self):
        self.buffers = 0
        self.start = 0

    def done(self):
        end = time.time()
        dt = end - self.start
        bps = self.buffers/dt
        spb = dt/self.buffers
        print '\t%d buffers / %fs\t= %f bps\t= %f spb' % (self.buffers, dt, bps, spb)

    def fakesrc(self, buffers):
        src = gst.element_factory_make('fakesrc','src')
        src.set_property('silent', 1)
        src.set_property('num_buffers', buffers)
        return src

    def fakesink(self):
        sink = gst.element_factory_make('fakesink','sink')
        sink.set_property('silent', 1)
        return sink

    def build_pipeline(self, buffers):
        pipeline = gst.Pipeline('pipeline')

        src = self.fakesrc(buffers)
        pipeline.add(src)
        sink = self.fakesink()
        pipeline.add(sink)
        src.link(sink)

        return pipeline

    def idle(self, pipeline):
        return pipeline.iterate()

    def test(self):
        self.bus = self.pipeline.get_bus()

        self.start = time.time()
        
        self.pipeline.set_state(gst.STATE_PLAYING)

        while 1:
            msg = self.bus.poll(gst.MESSAGE_EOS | gst.MESSAGE_ERROR, gst.SECOND)
            if msg:
                break

        self.pipeline.set_state(gst.STATE_NULL)
        self.done()

    def run(self, buffers):
        self.buffers = buffers
        
        print '# Testing buffer processing rate for "fakesrc ! fakesink"'
        print '# bps = buffers per second'
        print '# spb = seconds per buffer'
        
        self.pipeline = self.build_pipeline(buffers)
        assert self.pipeline

        self.test()
    
def main(args):
    "GStreamer Buffers-Per-Second tester"

    if len(args) < 2:
        print 'usage: %s buffers' % args[0]
        return 1
    
    bps = BPS()
    
    buffers = int(args[1])
    if buffers < 1:
        print 'buffers must be higher than 0'
        return

    bps.run(buffers)

if __name__ == '__main__':
    sys.exit(main(sys.argv))



#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# sinkelement.py
# (c) 2005 Edward Hervey <edward@fluendo.com>
# (c) 2007 Jan Schmidt <jan@fluendo.com>
# Licensed under LGPL
#
# Small test application to show how to write a sink element
# in 20 lines in python and place into the gstreamer registry
# so it can be autoplugged or used from parse_launch.
#
# You can run the example from the source doing from gst-python/:
#
#  $ export GST_PLUGIN_PATH=$GST_PLUGIN_PATH:$PWD/plugin:$PWD/examples/plugins
#  $ GST_DEBUG=python:4 gst-launch-1.0 fakesrc num-buffers=10 ! mysink

from gi.repository import Gst, GObject, GstBase
Gst.init(None)

#
# Simple Sink element created entirely in python
#
class MySink(GstBase.BaseSink):
    __gstmetadata__ = ('CustomSink','Sink', \
                      'Custom test sink element', 'Edward Hervey')

    __gsttemplates__ = Gst.PadTemplate.new("sink",
                                           Gst.PadDirection.SINK,
                                           Gst.PadPresence.ALWAYS,
                                           Gst.Caps.new_any())

    def do_render(self, buffer):
        Gst.info("timestamp(buffer):%s" % (Gst.TIME_ARGS(buffer.pts)))
        return Gst.FlowReturn.OK

GObject.type_register(MySink)
__gstelementfactory__ = ("mysink", Gst.Rank.NONE, MySink)



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2002 David I. Lehn
# Copyright (C) 2004 Johan Dahlin
# Copyright (C) 2005 Edward Hervey
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

import sys
from common import gst, unittest, TestCase

class CapsTest(TestCase):
    def setUp(self):
        TestCase.setUp(self)
        self.caps = gst.caps_from_string('video/x-raw-yuv,width=10,framerate=5/1;video/x-raw-rgb,width=15,framerate=10/1')
        self.assertEquals(self.caps.__refcount__, 1)
        self.structure = self.caps[0]
        self.any = gst.Caps("ANY")
        self.assertEquals(self.any.__refcount__, 1)
        self.empty = gst.Caps()
        self.assertEquals(self.empty.__refcount__, 1)

    def testCapsMime(self):
        mime = self.structure.get_name()
        assert mime == 'video/x-raw-yuv'

    def testCapsList(self):
        'check if we can access Caps as a list'
        structure = self.caps[0]
        mime = structure.get_name()
        assert mime == 'video/x-raw-yuv'
        structure = self.caps[1]
        mime = structure.get_name()
        assert mime == 'video/x-raw-rgb'

    def testCapsContainingMiniObjects(self):
        # buffer contains hex encoding of ascii 'abcd'
        caps = gst.Caps("video/x-raw-yuv, buf=(buffer)61626364")
        buf = caps[0]['buf']
        assert isinstance(buf, gst.Buffer)
        assert buf.data == "abcd"

        buf = gst.Buffer("1234")
        caps[0]['buf2'] = buf
        buf2 = caps[0]['buf2']
        assert buf2 == buf

    def testCapsConstructEmpty(self):
        caps = gst.Caps()
        assert isinstance(caps, gst.Caps)

    def testCapsConstructFromString(self):
        caps = gst.Caps('video/x-raw-yuv,width=10')
        assert isinstance(caps, gst.Caps)
        assert len(caps) == 1
        assert isinstance(caps[0], gst.Structure)
        assert caps[0].get_name() == 'video/x-raw-yuv'
        assert isinstance(caps[0]['width'], int)
        assert caps[0]['width'] == 10

    def testCapsConstructFromStructure(self):
        struct = gst.structure_from_string('video/x-raw-yuv,width=10,framerate=[0/1, 25/3]')
        caps = gst.Caps(struct)
        assert isinstance(caps, gst.Caps)
        assert len(caps) == 1
        assert isinstance(caps[0], gst.Structure)
        assert caps[0].get_name() == 'video/x-raw-yuv'
        assert isinstance(caps[0]['width'], int)
        assert caps[0]['width'] == 10
        assert isinstance(caps[0]['framerate'], gst.FractionRange)

    def testCapsConstructFromStructures(self):
        struct1 = gst.structure_from_string('video/x-raw-yuv,width=10')
        struct2 = gst.structure_from_string('video/x-raw-rgb,height=20.0')
        caps = gst.Caps(struct1, struct2)
        assert isinstance(caps, gst.Caps)
        assert len(caps) == 2
        struct = caps[0]
        assert isinstance(struct, gst.Structure), struct
        assert struct.get_name() == 'video/x-raw-yuv', struct.get_name()
        assert struct.has_key('width')
        assert isinstance(struct['width'], int)
        assert struct['width'] == 10
        struct = caps[1]
        assert isinstance(struct, gst.Structure), struct
        assert struct.get_name() == 'video/x-raw-rgb', struct.get_name()
        assert struct.has_key('height')
        assert isinstance(struct['height'], float)
        assert struct['height'] == 20.0

    def testCapsReferenceStructs(self):
        'test that shows why it\'s not a good idea to use structures by reference'
        caps = gst.Caps('hi/mom,width=0')
        structure = caps[0]
        del caps
        assert structure['width'] == 0
        

    def testCapsStructureChange(self):
        'test if changing the structure of the caps works by reference'
        assert self.structure['width'] == 10
        self.structure['width'] = 5
        assert self.structure['width'] == 5.0
        # check if we changed the caps as well
        structure = self.caps[0]
        assert structure['width'] == 5.0

    def testCapsBadConstructor(self):
        struct = gst.structure_from_string('video/x-raw-yuv,width=10')
        self.assertRaises(TypeError, gst.Caps, None)
        self.assertRaises(TypeError, gst.Caps, 1)
        self.assertRaises(TypeError, gst.Caps, 2.0)
        self.assertRaises(TypeError, gst.Caps, object)
        self.assertRaises(TypeError, gst.Caps, 1, 2, 3)
        
        # This causes segfault!
        #self.assertRaises(TypeError, gst.Caps, struct, 10, None)

    def testTrueFalse(self):
        'test that comparisons using caps work the intended way'
        assert self.any # not empty even though it has no structures
        assert not self.empty
        assert not gst.Caps('EMPTY') # also empty
        assert gst.Caps('your/mom')

    def testComparisons(self):
        assert self.empty < self.any
        assert self.empty < self.structure
        assert self.empty < self.caps
        assert self.caps < self.any
        assert self.empty <= self.empty
        assert self.caps <= self.caps
        assert self.caps <= self.any
        assert self.empty == "EMPTY"
        assert self.caps != self.any
        assert self.empty != self.any
        assert self.any > self.empty
        assert self.any >= self.empty

    def testFilters(self):
        name = 'video/x-raw-yuv'
        filtercaps = gst.Caps(*[struct for struct in self.caps if struct.get_name() == name])
        intersection = self.caps & 'video/x-raw-yuv'
        assert filtercaps == intersection

    def doSubtract(self, set, subset):
        '''mimic the test in GStreamer core's testsuite/caps/subtract.c'''
        assert not set - set
        assert not subset - subset
        assert not subset - set
        test = set - subset
        assert test
        test2 = test | subset
        test = test2 - set
        assert not test
        #our own extensions foolow here
        assert subset == set & subset
        assert set == set | subset
        assert set - subset == set ^ subset

    def testSubtract(self):
        self.doSubtract(
            gst.Caps ("some/mime, _int = [ 1, 2 ], list = { \"A\", \"B\", \"C\" }"),
            gst.Caps ("some/mime, _int = 1, list = \"A\""))
        self.doSubtract(
            gst.Caps ("some/mime, _double = (double) 1.0; other/mime, _int = { 1, 2 }"),
            gst.Caps ("some/mime, _double = (double) 1.0"))

    def testNoneValue(self):
        caps = gst.Caps("foo")
        
        def invalid_assignment():
            caps[0]["bar"] = None
        self.assertRaises(TypeError, invalid_assignment)

        def invalid_set_value():
            caps[0].set_value("bar", None)
        self.assertRaises(TypeError, invalid_set_value)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
# exampleTransform.py
# 2019 Daniel Klamt <graphics@pengutronix.de>

# Inverts a grayscale image in place, requires numpy.
#
# gst-launch-1.0 videotestsrc ! ExampleTransform ! videoconvert ! xvimagesink

import gi
gi.require_version('Gst', '1.0')
gi.require_version('GstBase', '1.0')
gi.require_version('GstVideo', '1.0')

from gi.repository import Gst, GObject, GstBase, GstVideo

import numpy as np

Gst.init(None)
FIXED_CAPS = Gst.Caps.from_string('video/x-raw,format=GRAY8,width=[1,2147483647],height=[1,2147483647]')

class ExampleTransform(GstBase.BaseTransform):
    __gstmetadata__ = ('ExampleTransform Python','Transform',
                      'example gst-python element that can modify the buffer gst-launch-1.0 videotestsrc ! ExampleTransform ! videoconvert ! xvimagesink', 'dkl')

    __gsttemplates__ = (Gst.PadTemplate.new("src",
                                           Gst.PadDirection.SRC,
                                           Gst.PadPresence.ALWAYS,
                                           FIXED_CAPS),
                       Gst.PadTemplate.new("sink",
                                           Gst.PadDirection.SINK,
                                           Gst.PadPresence.ALWAYS,
                                           FIXED_CAPS))

    def do_set_caps(self, incaps, outcaps):
        struct = incaps.get_structure(0)
        self.width = struct.get_int("width").value
        self.height = struct.get_int("height").value
        return True

    def do_transform_ip(self, buf):
        try:
            with buf.map(Gst.MapFlags.READ | Gst.MapFlags.WRITE) as info:
                # Create a NumPy ndarray from the memoryview and modify it in place:
                A = np.ndarray(shape = (self.height, self.width), dtype = np.uint8, buffer = info.data)
                A[:] = np.invert(A)

                return Gst.FlowReturn.OK
        except Gst.MapError as e:
            Gst.error("Mapping error: %s" % e)
            return Gst.FlowReturn.ERROR

GObject.type_register(ExampleTransform)
__gstelementfactory__ = ("ExampleTransform", Gst.Rank.NONE, ExampleTransform)



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2008 Alessandro Decina
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

from common import gst, unittest, TestCase, pygobject_2_13

import sys
import time

class TypeFindTest(TestCase):
    def testTypeFind(self):
        def application_awesome_type_find(typefind, arg1,  arg2):
            self.failUnlessEqual(arg1, 'arg1')
            self.failUnlessEqual(arg2, 'arg2')

            data = typefind.peek(0, 5)
            self.failUnless(data == '', 'peek out of length??')
            
            data = typefind.peek(0, 0)
            self.failUnless(data == '', '0 peek??')

            data = typefind.peek(3, 1)
            self.failUnless(data == 'M')

            data = typefind.peek(0, 4)
            self.failUnless(data == 'AWSM')

            typefind.suggest(gst.TYPE_FIND_MAXIMUM,
                    gst.Caps('application/awesome'))

        res = gst.type_find_register('application/awesome', gst.RANK_PRIMARY,
                application_awesome_type_find, ['.twi'],
                gst.Caps('application/awesome'), 'arg1', 'arg2')
        self.failUnless(res, 'type_find_register failed')

        factory = None
        factories = gst.type_find_factory_get_list()
        for typefind_factory in factories:
            if typefind_factory.get_name() == 'application/awesome':
                factory = typefind_factory
                break
        self.failUnless(factory is not None)

        obj = gst.Pad('src', gst.PAD_SRC)
        buffer = gst.Buffer('AWSM')
        caps, probability =  gst.type_find_helper_for_buffer(obj, buffer)

        self.failUnlessEqual(str(caps), 'application/awesome')
        self.failUnlessEqual(probability, gst.TYPE_FIND_MAXIMUM)



#!/usr/bin/env python

import pygtk
pygtk.require ("2.0")
import gobject
gobject.threads_init()

import pygst
pygst.require('0.10')
import gst

class PyIdentity(gst.Element):
    _sinkpadtemplate = gst.PadTemplate ("sink",
                                         gst.PAD_SINK,
                                         gst.PAD_ALWAYS,
                                         gst.caps_new_any())
    _srcpadtemplate = gst.PadTemplate ("src",
                                         gst.PAD_SRC,
                                         gst.PAD_ALWAYS,
                                         gst.caps_new_any())

    def __init__(self):
        gst.Element.__init__(self)

        self.sinkpad = gst.Pad(self._sinkpadtemplate, "sink")
        self.sinkpad.set_chain_function(self.chainfunc)
        self.sinkpad.set_event_function(self.eventfunc)
        self.sinkpad.set_getcaps_function(gst.Pad.proxy_getcaps)
        self.sinkpad.set_setcaps_function(gst.Pad.proxy_setcaps)
        self.add_pad (self.sinkpad)

        self.srcpad = gst.Pad(self._srcpadtemplate, "src")

        self.srcpad.set_event_function(self.srceventfunc)
        self.srcpad.set_query_function(self.srcqueryfunc)
        self.srcpad.set_getcaps_function(gst.Pad.proxy_getcaps)
        self.srcpad.set_setcaps_function(gst.Pad.proxy_setcaps)
        self.add_pad (self.srcpad)

    def chainfunc(self, pad, buffer):
        gst.log ("Passing buffer with ts %d" % (buffer.timestamp))
        return self.srcpad.push (buffer)

    def eventfunc(self, pad, event):
        return self.srcpad.push_event (event)
        
    def srcqueryfunc (self, pad, query):
        return self.sinkpad.query (query)
    def srceventfunc (self, pad, event):
        return self.sinkpad.push_event (event)

gobject.type_register(PyIdentity)

pipe = gst.Pipeline()
vt = gst.element_factory_make ("videotestsrc")
i1 = PyIdentity()
color = gst.element_factory_make ("ffmpegcolorspace")
scale = gst.element_factory_make ("videoscale")
q1 = gst.element_factory_make ("queue")
i2 = PyIdentity()
sink = gst.element_factory_make ("autovideosink")

pipe.add (vt, i1, q1, i2, color, scale, sink)
gst.element_link_many (vt, i1, q1, i2, color, scale, sink)

pipe.set_state (gst.STATE_PLAYING)

gobject.MainLoop().run()



#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

import sys

import pygtk
pygtk.require('2.0')

from gobject.option import OptionParser, OptionGroup
import pygst
pygst.require('0.10')

import gstoption

def main(args):
    parser = OptionParser()

    group = OptionGroup('flumotion', 'Flumotion options',
                        option_list=[])
    group.add_option('-v', '--verbose',
                      action="store_true", dest="verbose",
                      help="be verbose")
    group.add_option('', '--version',
                      action="store_true", dest="version",
                      default=False,
                      help="show version information")
    parser.add_option_group(group)

    parser.add_option_group(gstoption.get_group())

    options, args = parser.parse_args(args)

    if options.verbose:
        print 'Verbose mode'

    import gst

    if options.version:
        print sys.version, gst.version

if __name__ == '__main__':
    sys.exit(main(sys.argv))
